
class Vacuum:

    def __init__(self, environ, start=(0,0)):
        self._e = environ
        self._x = start[0]
        self._y = start[1]
        self._utility = 0

    @property
    def utility(self):
        return self._utility

    @utility.setter
    def utility(self, val):
        self._utility += val


class SimpleVacuum(Vacuum):

    def __init__(self):
        super(SimpleVacuum, self).__init__()
        self._x_path = []
        self._y_path = []

    def chooseMove(self):
        if self._e == 2:
            return 4

        actions = [0, 1, 2, 3, 6]
        if x == 1:
            actions.remove(0)
        if x == len(self._e)-2:
            actions.remove(1)
        if y == 1:
            actions.remove(2)
        if y == len(self._e)-2:
            actions.remove(3)

        return random.choice(actions)

    def move(self):
        if (action == 0):
            print("up")
            self._y -= 1
            self.utility = -1
        elif (action == 1):
            print("down")
            self._y += 1
            self.utility = -1
        elif (action == 2): # go left
            print("left")
            self._x -= 1
            self.utility = -1
        elif (action == 3): # go right
            print("right")
            self._x += 1
            self.utility = -1
        elif (action == 4): # clean
            print("clean")
            self._e[self._x][self._y] = 0
            self.utility = 10
        elif (action == 6): # idle
            print("idle")
            self.utility = 0

        self.path = (self._x, self._y)

    @property
    def x_path(self):
        return self._x_path

    @property
    def y_path(self):
        return self._y_path

    @path.setter
    def path(self, point):
        self._x_path.append(point[0])
        self._y_path.append(point[1])




class SearchPoint:
    def __init__(self, data, parent=None):
        self._data = data
        self._parent = parent

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, val):
        self._data = val

    @property
    def parent(self):
        return self._parent

    @parent.setter
    def parent(self, val):
        self._parent = val




import numpy as np

class Map2D:
    def __init__(self, m=10, n=10):
        self._x = m
        self._y = n
        self._map = np.zeros((m, n))

    def buildWall(self):
        self._map[:, 0] = 1
        self._map[0, :] = 1
        self._map[-1, :] = 1
        self._map[:, -1] = 1

    @property
    def map(self):
        return self._map


class Carpet(Map2D):
    
    def __init__(self):
        super(Carpet, self).__init__()
        self.randomizeDirt()
        self.generateDirt()

    def randomizeDirt(self):
        for x in range(1, self._x-1):
            for y in range(1, self._y-1):
                self._map[x, y] = random.uniform(0.1,0.6)

    def generateDirt(self):
        for x in range(1, self._x-1):
            for y in range(1, self._y-1):
                if (random.random()<self._map[x, y]):
                    self._map[x, y] = 2
                else:
                    self._map[x, y] = 0




import time


class TimerError(Exception):
    pass


class Timer:

    def __init__(self):
        self._start_time = None

    def start(self):
        if self._start_time is not None:
            raise TimerError(f"Timer is running. Use .stop() to stop it")
        self._start_time = time.perf_counter()

    def stop(self):
        if self._start_time is None:
            raise TimerError(f"Timer is not running. Use .start() to start it")
        elapsed_time = time.perf_counter() - self._start_time
        self._start_time = None
        print(f"Elapsed time: {elapsed_time:0.6f} seconds")

    def __enter__(self):
        """Start a new timer as a context manager"""
        self.start()
        return self

    def __exit__(self, *exc_info):
        """Stop the context manager timer"""
        self.stop()




class Location:
    def __init__(self, x=0, y=0, z=0):
        self._x = x
        self._y = y
        self._z = z

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def z(self):
        return self._z

    @x.setter
    def x(self, val):
        self._x = val

    @y.setter
    def y(self, val):
        self._y = val

    @z.setter
    def z(self, val):
        self._z = val




from cap6635.agents.blind.vacuum import SimpleVacuum
from cap6635.environment.map import Carpet


world = Carpet()
agent = SimpleVacuum(world)

import matplotlib.pyplot as plt
import matplotlib
matplotlib.use('TkAgg', force=True)


while True:
    plt.text(0,0,"Time Elapsed:%d; Utility: %.1f"%(timeElapsed,utility))
    plt.imshow(world, 'pink')
    plt.show(block=False)
    plt.plot(agent.x_path,agent.y_path,'r:',linewidth=1)
    plt.plot(agent.x_path[-1], agent.y_path[-1], '*r', 'Robot field', 5)
    plt.pause(0.5)
    agent.move()
    plt.clf()
